# Generated by `xclif compile` — do not edit by hand.
# Re-run `python -m xclif compile <routes_module>` after adding/removing routes.
from __future__ import annotations

from xclif import Cli


def _build_cli(version: str | None = None) -> Cli:
    from xclif_greeter.routes import _ as _root
    from xclif_greeter.routes.config import _ as _xclif_greeter_routes_config
    from xclif_greeter.routes.config.get import _ as _xclif_greeter_routes_config_get
    from xclif_greeter.routes.config.set import _ as _xclif_greeter_routes_config_set
    from xclif_greeter.routes.greet import _ as _xclif_greeter_routes_greet

    root = _root

    cli = Cli(root_command=root, version=version)
    cli.add_command(['config'], _xclif_greeter_routes_config)
    cli.add_command(['greet'], _xclif_greeter_routes_greet)
    cli.add_command(['config', 'get'], _xclif_greeter_routes_config_get)
    cli.add_command(['config', 'set'], _xclif_greeter_routes_config_set)
    return cli

